import lifeCycle from './lowcode/lifecycle'
import { app } from './app/weapps-api'
// 引入数据源管理器并进行初始化
import { setConfig, getAccessToken, EXTRA_API, createStateDataSourceVar, generateParamsParser } from './datasources/index';
import { redirectToLogin, findLoginPage, getAuthConfig } from './common/util'

// 设置数据源请求的 loading 及 toast 处理
setConfig({
  beforeDSRequest: (cfg) => {
    if (!cfg.options || !cfg.options.showLoading) return
    app.showLoading()
  },
  beforeCallFunction: async (params) => {
    try {
      const loginPage = findLoginPage();

      let skip = false;
      switch (params?.data?.methodName) {
        case 'callWedaApi': {
          if (['GetMiniProgramUserTicket', 'DescribeRuntimeResourceStrategy'].includes(params?.data?.params.action)) {
            skip = true;
          }
          break;
        }
      }

      // 后续做过滤处理
      if (!loginPage || (params?.data?.mode === 'c' && skip)) {
        return params;
      }
      // await initTcb();
      const { accessToken } = await getAccessToken();
      if (accessToken) {
        params.data.accessToken = accessToken;
      }
    } catch (e) {
      if (app?.cloud?.currentUser?.userType === "externalUser" && e?.error === 'unauthenticated') {
        app.showToast({
          title: '登录态失效',
          icon: 'error',
        });
      }
      console.error('beforeCallFunction error', e);
    }
    return params;
  },
  afterDSRequest: (cfg, error, result) => {
    if (!cfg.options) return
    if (cfg.options.showLoading) app.hideLoading()
    if (!cfg.options.showToast) return
    const isSuccess = !error && result && !result.code
    app.showToast({ icon: isSuccess ? 'success' : 'error' })
  },
  async afterCallFunction(params, error, res) {
    if (params?.data?.params?.action != 'DescribeRuntimeResourceStrategy' && ['InnerError.AuthFailure', 'InvalidAccessToken'].includes(res?.result?.code)) {
      const loginPage = findLoginPage();
      if (loginPage) {
        const authConfig = await getAuthConfig();
        let isAnonymous = true;
        try {
          const { accessToken } = await getAccessToken();
          isAnonymous = !accessToken;
        } catch (e) { }
        if (isAnonymous && authConfig.RejectStrategy == 'to_login') {
          redirectToLogin();
        } else if (authConfig.RejectStrategy == 'show_warning') {
          app.showToast({
            title: '接口无访问权限',
            icon: 'error',
          });
        }
      }
    }
  }
})

App({
  onLaunch(options) {
    this.app = app
    const onLaunch = lifeCycle.onLaunch || lifeCycle.onAppLaunch
    let { query = {} } = options
    EXTRA_API.setParams('$global', query)
    createStateDataSourceVar('$global', generateParamsParser({ app }))

    onLaunch && onLaunch.call(this, options)
      <% if (yyptConfig.yyptAppKey) { %>
        // 挂运营平台上报对象到app里
        app.yyptReport = wxReport
          <% }%>
  },
  onShow(options) {
    const fn = lifeCycle.onShow || lifeCycle.onAppShow
    fn && fn.call(this, options)
      <% if (yyptConfig.yyptAppKey) { %>
        wxReport.startReport()
        <% }%>
  },
  onHide() {
    const fn = lifeCycle.onHide || lifeCycle.onAppHide
    fn && fn.call(this)
  },
  onError(msg) {
    const fn = lifeCycle.onError || lifeCycle.onAppError
    fn && fn.call(this, msg)
  },
  onPageNotFound() {
    const fn = lifeCycle.onPageNotFound
    fn && fn.call(this)
  },
  onUnhandledRejection() {
    const fn = lifeCycle.onUnhandledRejection
    fn && fn.call(this)
  },
})
